// Copyright (c) 2022-2023 VMware Carbon Black, all rights reserved

// Package hinttype -- contains enumerations and methods related to hint generation
package hinttype

import (
	"encoding/json"
	"math/bits"
)

type HintSet uint64

func (hs *HintSet) UnmarshalJSON(b []byte) error {
	var keys []string
	err := json.Unmarshal(b, &keys)
	if err != nil {
		return err
	}

	hs.ConvertFromHintStringArray(keys)

	return nil
}

func (hs *HintSet) ConvertFromHintStringArray(hints []string) {
	for _, hintName := range hints {
		foundHint, found := ALL_HINT_STRINGS[hintName]
		if found {
			hs.SetHint(HintType(foundHint))
		}
	}
}

func (hs *HintSet) SetHint(hint HintType) {
	*hs |= HintSet(1 << hint)
}

func (hs *HintSet) IsHintSet(hint HintType) bool {
	val := *hs & HintSet(1<<hint)
	return val > 0
}

func (hs *HintSet) Count() int {
	return bits.OnesCount64(uint64(*hs))
}

func (hs *HintSet) MarshalJSON() ([]byte, error) {
	keys := hs.ToStringArray()
	return json.Marshal(keys)
}

func (hs *HintSet) ToStringArray() []string {
	hintCount := hs.Count()
	keys := make([]string, hintCount)
	keyIndex := 0
	for i := 0; i < NUM_HINTS; i++ {
		hint := HintType(i)
		if hs.IsHintSet(hint) {
			keys[keyIndex] = hint.String()
			keyIndex++
		}
	}
	return keys
}

func (hs *HintSet) ToIntArray() []int {
	hintCount := hs.Count()
	keys := make([]int, hintCount)
	keyIndex := 0
	for i := 0; i < NUM_HINTS; i++ {
		hint := HintType(i)
		if hs.IsHintSet(hint) {
			keys[keyIndex] = int(hint)
			keyIndex++
		}
	}
	return keys
}

func NewHintSet(hints ...HintType) *HintSet {
	hintMap := new(HintSet)
	for _, hintType := range hints {
		hintMap.SetHint(hintType)
	}
	return hintMap
}

func CompareHintSets(sourceHintSet, compareToHintSet *HintSet) bool {
	return *sourceHintSet == *compareToHintSet
}
